import unittest
from drafting.grid import Grid
from drafting.geometry import *
from drafting.text.reader import StyledString, Style

class TestText(unittest.TestCase):
    def test_glyph_name(self):
        style = Style("assets/ColdtypeObviously-VF.ttf")
        ss = StyledString("CDELOPTY", style)
        ssps = ss.pens()
        self.assertEqual(len(ssps), 8)
        self.assertEqual(ssps[0].glyphName, "C")
        self.assertEqual(ssps[-1].glyphName, "Y")

if __name__ == "__main__":
    unittest.main()


import asyncio
import functools
import logging


def asyncTask(func):
    """Wraps an async function or method into a regular function, that
    will schedule the async function as a task. Returns an asyncio Task
    object, that can be cancelled.
    """
    @functools.wraps(func)
    def createFuncTask(*args, **kwargs):
        coro = func(*args, **kwargs)
        task = asyncio.create_task(coro)
        task.add_done_callback(_done_callback)
        return task
    return createFuncTask


def asyncTaskAutoCancel(func):
    """Wraps an async method into a regular method, that will schedule
    the async function as a task. If this task has previously been scheduled
    and has not yet run, it will be cancelled. So a newly scheduled task
    overrides an older one.
    """
    taskAttributeName = f"_{func.__name__}_autoCancelTask"
    @functools.wraps(func)
    def createFuncTask(self, *args, **kwargs):
        oldTask = getattr(self, taskAttributeName, None)
        if oldTask is not None:
            oldTask.cancel()
        coro = func(self, *args, **kwargs)
        task = asyncio.create_task(coro)
        task.add_done_callback(_done_callback)
        setattr(self, taskAttributeName, task)
        return task
    return createFuncTask


def _done_callback(task):
    if task.cancelled():
        return
    if task.exception() is not None:
        task.print_stack()


def suppressAndLogException(func):
    """Wraps a method or function into a try/except, logging any errors while
    silencing them. This is handy for debugging Cocoa methods, which often
    cause trouble if they trow an exception.

    When an exception occurs, the decorated function will return None, so only
    use this decorator if returning None is a valid option.
    """

    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception:
            logging.exception(func.__name__)

    return wrapper



#from drafting.text.reader import Style, StyledString, SegmentedString, normalize_font_path, Font, FontNotFoundException
#from drafting.text.composer import Slug, Lockup, Graf, GrafStyle, T2L, Composer


# pip install git+https://github.com/typemytype/drawbot

import drawBot as db

from drafting.geometry import Rect, Point
from drafting.color import Color, Gradient, hsl, rgb
from drafting.pens.draftingpens import DraftingPen, DraftingPens


def db_fill(color):
    if color:
        if isinstance(color, Gradient):
            db_gradient(color)
        elif isinstance(color, Color):
            db.fill(color.r, color.g, color.b, color.a)
    else:
        db.fill(None)

def db_stroke(weight=1, color=None, dash=None):
    db.strokeWidth(weight)
    if dash:
        db.lineDash(dash)
    if color:
        if isinstance(color, Gradient):
            pass # possible?
        elif isinstance(color, Color):
            db.stroke(color.r, color.g, color.b, color.a)
    else:
        db.stroke(None)

def db_shadow(clip=None, radius=10, alpha=0.3, color=Color.from_rgb(0,0,0,1)):
    if clip:
        bp = db.BezierPath()
        bp.rect(*clip)
        db.clipPath(bp)
    #elif self.rect:
    #    cp = DATPen(fill=None).rect(self.rect).xor(self.dat)
    #    bp = db.BezierPath()
    #    cp.replay(bp)
    #    db.clipPath(bp)
    db.shadow((0, 0), radius*3, list(color.with_alpha(alpha)))
    
def db_gradient(gradient):
    stops = gradient.stops
    db.linearGradient(stops[0][1], stops[1][1], [list(s[0]) for s in stops], [0, 1])


class DrawBotPen(DraftingPen):
    def bezierPath(self):
        bp = db.BezierPath()
        self.replay(bp)
        return bp

    def _draw(self):
        for dp in list(self.all_pens()):
            with db.savedState():
                db.fill(None)
                db.stroke(None)
                db.strokeWidth(0)
                for attr, value in dp.allStyledAttrs().items():
                    if attr == "fill":
                        db_fill(value)
                    elif attr == "stroke":
                        db_stroke(value.get("weight", 1), value.get("color"), None)
                    db.drawPath(dp.cast(DrawBotPen).bezierPath())
        return self
    
    def draw(self, rect=None, filters=[]):
        if rect and len(filters) > 0:
            im = db.ImageObject()
            with im:
                db.size(*rect.wh())
                self._draw()
            for filter_name, filter_kwargs in filters:
                getattr(im, filter_name)(**filter_kwargs)
            x, y = im.offset()
            db.image(im, (x, y))
        else:
            self._draw()
        return self